/**
 * @file
 * Provides Zooming loader.
 */

(function ($, Drupal) {

  'use strict';

  /**
   * Toggle zooming classes.
   *
   * @param {HTMLElement} target
   *   The target HTML element.
   * @param {Bool} isOpen
   *   If it is zooming.
   */
  function toggleClasses(target, isOpen) {
    $('body')[isOpen ? 'addClass' : 'removeClass']('is-zooming');

    // @todo configurable selectors.
    // The trouble is Zooming element will be cropped if its parent has
    // `overflow: hidden` rule. This releases the potential unwanted crop.
    // Use your CSS with `.is-zooming` class to release cropping temporarily.
    var sels = '.box, .content, .field-item, .grid, .slide, .views-row';
    $(target).closest(sels)[isOpen ? 'addClass' : 'removeClass']('is-zooming__zoomed');
  }

  /**
   * Zooming utility functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The Zooming gallery container HTML element.
   */
  function doZooming(i, elm) {
    var options = {
      // Avoid too small as too early release.
      scrollThreshold: 210,
      onImageLoading: function (target) {
        toggleClasses(target, true);
        $('body').addClass('is-zooming--loading');
      },
      onImageLoaded: function (target) {
        $('body').removeClass('is-zooming--loading');
      },
      onOpen: function (target) {
        toggleClasses(target, true);
        $('body').addClass('is-zooming--open');
      },
      onClose: function (target) {
        toggleClasses(target, false);
        $('body').removeClass('is-zooming--open');

        // Fix for erratic image transforms within grids.
        $(target).removeAttr('style');
      }
    };

    var zooming = new Zooming(options).listen('.zooming');

    // Allows styling the overlay by adding proper CSS class.
    $(zooming.overlay.el).addClass('is-zooming__overlay');
  }

  /**
   * Attaches zooming behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.zooming = {
    attach: function (context) {
      $('[data-zooming-gallery], .zooming-gallery', context).once('zooming-gallery', doZooming);
    }
  };

})(jQuery, Drupal);
